@@ -620,15 +620,29 @@ void KArchiveTest::testTarCopyTo()
 void KArchiveTest::testTarReadWrite()
 {
     QFETCH(QString, fileName);
+    QFETCH(QString, mimeType);
 
     // testCreateTar must have been run first.
-    KTar tar(fileName);
-    QVERIFY(tar.open(QIODevice::ReadWrite));
+    {
+        KTar tar(fileName);
+        QVERIFY(tar.open(QIODevice::ReadWrite));
 
-    testReadWrite(&tar);
-    testFileData(&tar);
+        testReadWrite(&tar);
+        testFileData(&tar);
 
-    QVERIFY(tar.close());
+        QVERIFY(tar.close());
+    }
+
+    {
+        // Now test with KCompressionDevice
+        KCompressionDevice kd(std::make_unique<QFile>(fileName), KCompressionDevice::compressionTypeForMimeType(mimeType));
+        KTar tar(&kd);
+        QVERIFY(tar.open(QIODevice::ReadOnly));
+
+        testFileData(&tar);
+
+        QVERIFY(tar.close());
+    }
 
     // Reopen it and check it
     {
@@ -899,6 +913,15 @@ void KArchiveTest::testTarDirectoryTwice() // bug 206994
     QCOMPARE(listing.count(), 3);
 }
 
+void KArchiveTest::testTarGzHugeMemoryUsage()
+{
+    const QString filePath = QFINDTESTDATA("data/ossfuzz_testcase_6581632288227328.tar.gz");
+    QVERIFY(!filePath.isEmpty());
+
+    KTar tar(filePath);
+    QVERIFY(!tar.open(QIODevice::ReadOnly));
+}
+
 void KArchiveTest::testTarIgnoreRelativePathOutsideArchive()
 {
 #if HAVE_BZIP2_SUPPORT
@@ -924,6 +947,36 @@ void KArchiveTest::testTarIgnoreRelativePathOutsideArchive()
     QSKIP("Test data is in bz2 format and karchive is built without bzip2 format");
 #endif
 }
+
+void KArchiveTest::testTarDeepDirHierarchy()
+{
+#if HAVE_XZ_SUPPORT
+    const QString fileName = QFINDTESTDATA("data/tar_deep_hierarchy.tar.xz");
+    QVERIFY(!fileName.isEmpty());
+
+    KTar tar(fileName);
+    QVERIFY(tar.open(QIODevice::ReadOnly));
+    traverseArchive(tar.directory());
+#else
+    QSKIP("Test data is in xz format and karchive is built without xz format");
+#endif
+}
+
+void KArchiveTest::benchmarkTarDeepDirHierarchy()
+{
+#if HAVE_XZ_SUPPORT
+    const QString fileName = QFINDTESTDATA("data/tar_deep_hierarchy.tar.xz");
+    QVERIFY(!fileName.isEmpty());
+
+    QBENCHMARK {
+        KTar tar(fileName);
+        QVERIFY(tar.open(QIODevice::ReadOnly));
+    }
+#else
+    QSKIP("Test data is in xz format and karchive is built without xz format");
+#endif
+}
+
 ///
 
 static const char s_zipFileName[] = "karchivetest.zip";
@@ -1563,6 +1616,27 @@ void KArchiveTest::testZipOssFuzzIssue433303801()
     traverseArchive(zip.directory());
 }
 
+void KArchiveTest::testZipDeepDirHierarchy()
+{
+    const QString fileName = QFINDTESTDATA("data/zip_deep_hierarchy.zip");
+    QVERIFY(!fileName.isEmpty());
+
+    KZip zip(fileName);
+    QVERIFY(zip.open(QIODevice::ReadOnly));
+    traverseArchive(zip.directory());
+}
+
+void KArchiveTest::benchmarkZipDeepDirHierarchy()
+{
+    const QString fileName = QFINDTESTDATA("data/zip_deep_hierarchy.zip");
+    QVERIFY(!fileName.isEmpty());
+
+    QBENCHMARK {
+        KZip zip(fileName);
+        QVERIFY(zip.open(QIODevice::ReadOnly));
+    }
+}
+
 void KArchiveTest::testRcc()
 {
     const QString rccFile = QFINDTESTDATA("data/runtime_resource.rcc"); // was copied from qtbase/tests/auto/corelib/io/qresourceengine
@@ -1753,13 +1827,15 @@ void KArchiveTest::test7ZipReadWrite()
     QFETCH(QString, fileName);
 
     // testCreate7zip must have been run first.
-    K7Zip k7zip(fileName);
-    QVERIFY(k7zip.open(QIODevice::ReadWrite));
+    {
+        K7Zip k7zip(fileName);
+        QVERIFY(k7zip.open(QIODevice::ReadWrite));
 
-    testReadWrite(&k7zip);
-    testFileData(&k7zip);
+        testReadWrite(&k7zip);
+        testFileData(&k7zip);
 
-    QVERIFY(k7zip.close());
+        QVERIFY(k7zip.close());
+    }
 
     // Reopen it and check it
     {
@@ -1901,13 +1977,29 @@ void KArchiveTest::test7ZipOssFuzzIssues_data()
     QTest::newRow("issue_440829292") << "data/ossfuzz_issue_440829292.7z";
     QTest::newRow("issue_441906077") << "data/ossfuzz_issue_441906077.7z";
     QTest::newRow("testcase_4566647131406336") << "data/ossfuzz_testcase_4566647131406336.7z";
+    QTest::newRow("testcase_4784793343819776") << "data/ossfuzz_testcase_4784793343819776.7z";
+    QTest::newRow("testcase_5082404562993152") << "data/ossfuzz_testcase_5082404562993152.7z";
+    QTest::newRow("testcase_5125023070486528") << "data/ossfuzz_testcase_5125023070486528.7z";
+    QTest::newRow("testcase_5132547098214400") << "data/ossfuzz_testcase_5132547098214400.7z";
+    QTest::newRow("testcase_5141245598171136") << "data/ossfuzz_testcase_5141245598171136.7z";
+    QTest::newRow("testcase_5161502865948672") << "data/ossfuzz_testcase_5161502865948672.7z";
+    QTest::newRow("testcase_5497856632094720") << "data/ossfuzz_testcase_5497856632094720.7z";
+    QTest::newRow("testcase_5503074996387840") << "data/ossfuzz_testcase_5503074996387840.7z";
     QTest::newRow("testcase_5560695602348032") << "data/ossfuzz_testcase_5560695602348032.7z";
     QTest::newRow("testcase_5581009572921344") << "data/ossfuzz_testcase_5581009572921344.7z";
     QTest::newRow("testcase_5821682553257984") << "data/ossfuzz_testcase_5821682553257984.7z";
+    QTest::newRow("testcase_5991129817612288") << "data/ossfuzz_testcase_5991129817612288.7z";
     QTest::newRow("testcase_6077171694370816") << "data/ossfuzz_testcase_6077171694370816.7z";
     QTest::newRow("testcase_6096742417498112") << "data/ossfuzz_testcase_6096742417498112.7z";
+    QTest::newRow("testcase_6213340184772608") << "data/ossfuzz_testcase_6213340184772608.7z";
+    QTest::newRow("testcase_6248361801089024") << "data/ossfuzz_testcase_6248361801089024.7z";
+    QTest::newRow("testcase_6366650283917312") << "data/ossfuzz_testcase_6366650283917312.7z";
+    QTest::newRow("testcase_6532014901886976") << "data/ossfuzz_testcase_6532014901886976.7z";
     QTest::newRow("testcase_6593198198947840") << "data/ossfuzz_testcase_6593198198947840.7z";
+    QTest::newRow("testcase_6597581431177216") << "data/ossfuzz_testcase_6597581431177216.7z";
+    QTest::newRow("testcase_6649925879529472") << "data/ossfuzz_testcase_6649925879529472.7z";
     QTest::newRow("testcase_6653406484955136") << "data/ossfuzz_testcase_6653406484955136.7z";
+    QTest::newRow("testcase_6740459604213760") << "data/ossfuzz_testcase_6740459604213760.7z";
 }
 
 void KArchiveTest::test7ZipOssFuzzIssues()
@@ -1955,6 +2047,52 @@ void KArchiveTest::test7ZipPasswordProtected()
 }
 #endif
 
+void KArchiveTest::test7ZipReadCoder_data()
+{
+    QTest::addColumn<QString>("fileName");
+
+    QTest::newRow("copy") << "data/7z_coder_test_copy.7z";
+    QTest::newRow("lzma") << "data/7z_coder_test_lzma.7z";
+    QTest::newRow("lzma2") << "data/7z_coder_test_lzma2.7z";
+    QTest::newRow("deflate") << "data/7z_coder_test_deflate.7z";
+    QTest::newRow("bcj") << "data/7z_coder_test_bcj.7z";
+    QTest::newRow("bcj2") << "data/7z_coder_test_bcj2.7z";
+    QTest::newRow("bzip2") << "data/7z_coder_test_bzip2.7z";
+    QTest::newRow("zstd") << "data/7z_coder_test_zstd.7z";
+}
+
+/*!
+ * @dataProvider test7ZipReadCoder_data
+ */
+void KArchiveTest::test7ZipReadCoder()
+{
+    QFETCH(QString, fileName);
+
+    const QString filePath = QFINDTESTDATA(fileName);
+    QVERIFY(!filePath.isEmpty());
+
+    K7Zip k7zip(filePath);
+    QVERIFY(k7zip.open(QIODevice::ReadOnly));
+
+    const KArchiveDirectory *dir = k7zip.directory();
+    QVERIFY(dir != nullptr);
+
+    const KArchiveEntry *entry = dir->entry("hello.txt");
+    QVERIFY(entry != nullptr);
+    QVERIFY(entry->isFile());
+
+    const QByteArray fileData = QByteArray("Hello, World!\n");
+
+    const KArchiveFile *file = static_cast<const KArchiveFile *>(entry);
+
+    if (fileName.contains("bcj2")) {
+        QEXPECT_FAIL("", "bcj2 decoding fails, see https://bugs.kde.org/show_bug.cgi?id=509201", Continue);
+    }
+    QCOMPARE(file->data(), fileData);
+
+    QVERIFY(k7zip.close());
+}
+
 #endif
 
 #include "moc_karchivetest.cpp"
