@@ -12,11 +12,11 @@
 #include <QBuffer>
 #include <QDir>
 #include <QDirIterator>
+#include <QList>
 #include <QNetworkReply>
 #include <QNetworkRequest>
 #include <QTemporaryDir>
 #include <QTest>
-#include <QVector>
 
 QTEST_MAIN(KCompressionDeviceTest)
 
@@ -44,6 +44,8 @@ QString KCompressionDeviceTest::formatExtension(KCompressionDevice::CompressionT
         return "tar.gz";
     case KCompressionDevice::BZip2:
         return "tar.bz2";
+    case KCompressionDevice::Lz:
+        return "tar.lz";
     case KCompressionDevice::Xz:
         return "tar.xz";
     case KCompressionDevice::Zstd:
@@ -90,15 +92,16 @@ void KCompressionDeviceTest::testExtraction()
     QVERIFY(QDir("examples/tarlocalfiles").exists());
     QVERIFY(QDir("examples/unzipper").exists());
 
-    const QStringList fileList = {QStringLiteral("examples/bzip2gzip/CMakeLists.txt"),
-                                  QStringLiteral("examples/bzip2gzip/main.cpp"),
-                                  QStringLiteral("examples/helloworld/CMakeLists.txt"),
-                                  QStringLiteral("examples/helloworld/helloworld.pro"),
-                                  QStringLiteral("examples/helloworld/main.cpp"),
-                                  QStringLiteral("examples/tarlocalfiles/CMakeLists.txt"),
-                                  QStringLiteral("examples/tarlocalfiles/main.cpp"),
-                                  QStringLiteral("examples/unzipper/CMakeLists.txt"),
-                                  QStringLiteral("examples/unzipper/main.cpp")};
+    const QStringList fileList = {
+        QStringLiteral("examples/bzip2gzip/CMakeLists.txt"),
+        QStringLiteral("examples/bzip2gzip/main.cpp"),
+        QStringLiteral("examples/helloworld/CMakeLists.txt"),
+        QStringLiteral("examples/helloworld/main.cpp"),
+        QStringLiteral("examples/tarlocalfiles/CMakeLists.txt"),
+        QStringLiteral("examples/tarlocalfiles/main.cpp"),
+        QStringLiteral("examples/unzipper/CMakeLists.txt"),
+        QStringLiteral("examples/unzipper/main.cpp"),
+    };
 
     for (const QString &s : fileList) {
         QFileInfo extractedFile(s);
@@ -132,6 +135,15 @@ void KCompressionDeviceTest::testBZip2BufferedDevice()
 #endif
 }
 
+void KCompressionDeviceTest::testLzBufferedDevice()
+{
+#ifdef HAVE_LZIP_SUPPORT_FILE
+    testBufferedDevice(KCompressionDevice::Lz);
+#else
+    QSKIP("This test needs lzip support");
+#endif
+}
+
 void KCompressionDeviceTest::testXzBufferedDevice()
 {
 #if HAVE_XZ_SUPPORT
@@ -153,7 +165,7 @@ void KCompressionDeviceTest::testZstdBufferedDevice()
 void KCompressionDeviceTest::testWriteErrorOnOpen()
 {
     // GIVEN
-    QString fileName("/I/dont/exist/kcompressiondevicetest-write.gz");
+    QString fileName("/I/do/not/exist/kcompressiondevicetest-write.gz");
     KCompressionDevice dev(fileName, KCompressionDevice::GZip);
     // WHEN
     QVERIFY(!dev.open(QIODevice::WriteOnly));
